<?php
namespace Ep\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Detektira osnovni URL aplikacije.
 *
 * @package Ep\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
final class BaseUrl implements MiddlewareInterface
{
    const BASE_URL = 'BaseUrl';
    const BASE_PATH = 'BasePath';

    /** @var string */
    protected $baseUrl;

    /**
     * Vraća osnovni url aplikacije.
     *
     * @return string
     */
    public function getBaseUrl(): string
    {
        if (null == $this->baseUrl) {
            $this->baseUrl = $this->detectBaseUrl();
        }

        return $this->baseUrl;
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $uri = $request->getUri();
        $uriPath = $uri->getPath();

        $this->baseUrl = $baseUrl = $this->detectBaseUrl($request);
        $basePath      = $this->detectBasePath($request, $baseUrl);

        $request = $request
            ->withAttribute(self::BASE_URL, $baseUrl)
            ->withAttribute(self::BASE_PATH, $basePath);

        if (!empty($baseUrl) && strpos($uriPath, $baseUrl) === 0) {
            $path = substr($uriPath, strlen($baseUrl));
            $path = sprintf('/%s', ltrim($path, '/'));
            /** @noinspection PhpUndefinedMethodInspection */
            $request = $request->withUri($uri->withPath($path));
        }

        return $handler->handle($request);
    }

    /**
     * Pokušava otkriti koji dio putanje requesta je stvaran.
     *
     * @param ServerRequestInterface $request
     *
     * @return string
     */
    private function detectBaseUrl(ServerRequestInterface $request = null): string
    {
        $server = $request ? $request->getServerParams() : [];

        $filename = $server['SCRIPT_FILENAME'] ?? ($_SERVER['SCRIPT_FILENAME'] ?? '');
        $scriptName = $server['SCRIPT_NAME'] ?? ($_SERVER['SCRIPT_NAME'] ?? '');
        $phpSelf = $server['PHP_SELF'] ?? ($_SERVER['PHP_SELF'] ?? '');
        $origScriptName = $server['ORIG_SCRIPT_NAME'] ?? ($_SERVER['ORIG_SCRIPT_NAME'] ?? '');

        if ($scriptName !== null && basename($scriptName) === $filename) {
            $baseUrl = $scriptName;
        } elseif ($phpSelf !== null && basename($phpSelf) === $filename) {
            $baseUrl = $phpSelf;
        } elseif ($origScriptName !== null && basename($origScriptName) === $filename) {
            // 1and1 shared hosting compatibility.
            $baseUrl = $origScriptName;
        } else {
            // Backtrack up the SCRIPT_FILENAME to find the portion
            // matching PHP_SELF.

            $baseUrl  = '/';
            $basename = basename($filename);
            if ($basename) {
                $path     = ($phpSelf ? trim($phpSelf, '/') : '');
                $basePos  = strpos($path, $basename) ?: 0;
                $baseUrl .= substr($path, 0, $basePos) . $basename;
            }
        }

        // If the baseUrl is empty, then simply return it.
        if (empty($baseUrl)) {
            return '';
        }

        // Does the base URL have anything in common with the request URI?
        $requestUri = $server['REQUEST_URI'] ?? ($_SERVER['REQUEST_URI'] ?? '');

        // Full base URL matches.
        if (0 === strpos($requestUri, $baseUrl)) {
            return dirname($baseUrl);
        }

        // Directory portion of base path matches.
        $baseDir = str_replace('\\', '/', dirname($baseUrl));
        if (0 === strpos($requestUri, $baseDir)) {
            return $baseDir;
        }

        $truncatedRequestUri = $requestUri;

        if (false !== ($pos = strpos($requestUri, '?'))) {
            $truncatedRequestUri = substr($requestUri, 0, $pos);
        }

        $basename = basename($baseUrl);

        // No match whatsoever
        if (empty($basename) || false === strpos($truncatedRequestUri, $basename)) {
            return '';
        }

        // If using mod_rewrite or ISAPI_Rewrite strip the script filename
        // out of the base path. $pos !== 0 makes sure it is not matching a
        // value from PATH_INFO or QUERY_STRING.
        if (strlen($requestUri) >= strlen($baseUrl)
            && (false !== ($pos = strpos($requestUri, $baseUrl)) && $pos !== 0)
        ) {
            $baseUrl = substr($requestUri, 0, $pos + strlen($baseUrl));
        }

        $baseUrl = preg_replace('#/[^/]+\.php$#', '', $baseUrl);

        return $baseUrl;
    }

    /**
     * Vraća direktorij u kojem se aplikacija nalazi.
     *
     * @param ServerRequestInterface $request
     * @param string $baseUrl
     *
     * @return string
     */
    private function detectBasePath(ServerRequestInterface $request, string $baseUrl): string
    {
        if ($baseUrl === '') {
            return '';
        }

        $filename = basename($request->getServerParams()['SCRIPT_FILENAME'] ?? '');

        if (basename($baseUrl) === $filename) {
            return str_replace('\\', '/', dirname($baseUrl));
        }

        return $baseUrl;
    }
}