<?php
namespace EpTest\Middleware;

use Ep\ConfigProvider;
use Ep\Middleware\ForceWww;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use Psr\Http\Server\MiddlewareInterface;
use ReflectionClass;
use SplQueue;
use Zend\ConfigAggregator\ArrayProvider;
use Zend\ConfigAggregator\ConfigAggregator;
use Zend\Diactoros\ServerRequestFactory;
use Zend\ServiceManager\ServiceManager;
use Zend\Stratigility\MiddlewarePipe;
use Zend\Stratigility\MiddlewarePipeInterface;

/**
 * Class AppTest
 *
 * @package EpTest\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AppTest extends TestCase
{
    use ContainerAwareTrait;

    /** @var string */
    protected $cwd;

    /**
     * Postavljanje testa.
     */
    protected function setUp()
    {
        parent::setUp();

        // spremamo trenutni dir za kasnije.
        $this->cwd = getcwd();
        chdir(__DIR__);

        $this->rrmdir('var');
    }

    public function rrmdir($path){
        if (is_dir($path)) {
            array_map( [$this, "rrmdir"], glob($path . DIRECTORY_SEPARATOR . '{,.[!.]}*', GLOB_BRACE) );
            @rmdir($path);
        }
        else {
            @unlink($path);
        }
    }

    /**
     * Čišćenje nakon testa.
     */
    protected function tearDown()
    {
        parent::tearDown();

        // brišemo datoteke koje napravimo tijekom testova.
        $filesToDelete = [
            'var/log/app.log',
            'var/log',
            'var'
        ];

        foreach ($filesToDelete as $file) {
            if (is_dir($file)) {
                $this->rrmdir($file);
            } else if (file_exists($file)) {
                unlink($file);
            }
        }

        // vraćamo cwd gdje smo bili.
        chdir($this->cwd);
    }


    public function testAppLoggerWillCreateVarLog()
    {
        $container = $this->createContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        $container->get('app.logger');

        $this->assertTrue(file_exists('var/log/app.log'));
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     * @throws \ReflectionException
     */
    public function testMiddlwareIsInjectedIntoTheApplication()
    {
        $container = $this->createContainer([
            'app' => [
                'middleware' => [
                    'app.force_www',
                ],
            ],
        ]);

        /** @var MiddlewarePipe $app */
        $app = $container->get('app');

        $class = new ReflectionClass($app);
        $pipelineProperty = $class->getProperty('pipeline');
        $pipelineProperty->setAccessible(true);
        /** @var SplQueue $pipeline */
        $pipeline = $pipelineProperty->getValue($app);

        $found = false;
        foreach ($pipeline as $middleware) {
            if ($middleware instanceof ForceWww) {
                $found = true;
                break;
            }
        }

        $this->assertTrue($found);
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     * @throws \ReflectionException
     */
    public function testMiddlewareCanBeDisabledInConfigOverrides()
    {
        $container = $this->createContainer([
            'app' => [
                'middleware' => [
                    'app.force_www',
                    'app.force_www' => false,
                ],
            ],
        ]);

        /** @var MiddlewarePipe $app */
        $app = $container->get('app');

        $class = new ReflectionClass($app);
        $pipelineProperty = $class->getProperty('pipeline');
        $pipelineProperty->setAccessible(true);
        /** @var SplQueue $pipeline */
        $pipeline = $pipelineProperty->getValue($app);

        $found = false;

        foreach ($pipeline as $middleware) {
            if ($middleware instanceof ForceWww) {
                $found = true;
                break;
            }
        }

        $this->assertFalse($found);
    }

    /**
     * @throws \ReflectionException
     */
    public function testMiddlewareIsNotRepeatedEver()
    {
        $config = (new ConfigAggregator([
            ConfigProvider::class,
            ConfigProvider::class,
            new ArrayProvider([
                'app' => [
                    'development' => false,
                ],
            ]),
            new ArrayProvider([
                'app' => [
                    'middleware' => [
                        'app.error_handler',
                        'app.error_handler',
                    ],
                ],
            ]),
        ]))->getMergedConfig();

        $config = (new ConfigAggregator([
            ConfigProvider::class,
            ConfigProvider::class,
            new ArrayProvider([
                'app' => [
                    'development' => true,
                ],
            ]),

            new ArrayProvider($config),

            new ArrayProvider([
                'app' => [
                    'middleware' => [
                        'app.error_handler',
                        'app.error_handler',
                    ],
                ],
            ]),
        ]))->getMergedConfig();

        $container = new ServiceManager($config['dependencies']);
        $container->setService('config', $config);

        $app = $container->get('app');

        $class = new ReflectionClass($app);
        $pipelineProperty = $class->getProperty('pipeline');
        $pipelineProperty->setAccessible(true);
        /** @var SplQueue $pipeline */
        $pipeline = $pipelineProperty->getValue($app);

        $middlewares = [];

        foreach ($pipeline as $middleware) {
            $middlewares[] = get_class($middleware);
        }

        $this->assertEquals(count($middlewares), count(array_unique($middlewares)));
    }

    /**
     * TODO: Ovaj test ne prolazi na gitlabu ... WTF?
     */
    public function MiddlewareIsRegisteredToPath()
    {
        $middleware = $this->createMock(MiddlewareInterface::class);
        $middleware
            ->expects($this->once())
            ->method('process');

        $container = $this->createContainer([
            'app' => [
                'middleware' => [
                    '/some-path' => 'mock_middleware',
                ],
            ],
        ]);

        $container->setService('mock_middleware', $middleware);
        /** @var MiddlewarePipeInterface $app */
        $app = $container->get('app');

        $request = ServerRequestFactory::fromGlobals();
        $uri = $request->getUri()->withPath('/some-path');
        $request = $request->withUri($uri);

        $app->handle($request);
    }
}